@@ -1,9 +1,10 @@
 // Copyright (c) 2020 Red Hat, Inc.
+// Copyright Contributors to the Open Cluster Management project
 
 package apis
 
 import (
-	v1 "github.com/stolostron/config-policy-controller/pkg/apis/policy/v1"
+	v1 "github.com/open-cluster-management/config-policy-controller/pkg/apis/policy/v1"
 )
 
 func init() {
