@@ -1,11 +1,12 @@
 // Copyright (c) 2020 Red Hat, Inc.
+// Copyright Contributors to the Open Cluster Management project
 
 package common
 
 import (
 	"sync"
 
-	policiesv1 "github.com/stolostron/config-policy-controller/pkg/apis/policy/v1"
+	policiesv1 "github.com/open-cluster-management/config-policy-controller/pkg/apis/policy/v1"
 )
 
 //SyncedPolicyMap a thread safe map
