@@ -1,9 +1,10 @@
 // Copyright (c) 2020 Red Hat, Inc.
+// Copyright Contributors to the Open Cluster Management project
 
 package controller
 
 import (
-	configurationpolicy "github.com/stolostron/config-policy-controller/pkg/controller/configurationpolicy"
+	configurationpolicy "github.com/open-cluster-management/config-policy-controller/pkg/controller/configurationpolicy"
 )
 
 func init() {
