@@ -1,4 +1,5 @@
 // Copyright (c) 2020 Red Hat, Inc.
+// Copyright Contributors to the Open Cluster Management project
 
 package configurationpolicy
 
@@ -15,8 +16,9 @@ import (
 
 	"github.com/golang/glog"
 	gocmp "github.com/google/go-cmp/cmp"
-	policyv1 "github.com/stolostron/config-policy-controller/pkg/apis/policy/v1"
-	common "github.com/stolostron/config-policy-controller/pkg/common"
+	policyv1 "github.com/open-cluster-management/config-policy-controller/pkg/apis/policy/v1"
+	common "github.com/open-cluster-management/config-policy-controller/pkg/common"
+	templates "github.com/open-cluster-management/config-policy-controller/pkg/common/templates"
 	corev1 "k8s.io/api/core/v1"
 	rbacv1 "k8s.io/api/rbac/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
@@ -125,6 +127,7 @@ func Initialize(kubeconfig *rest.Config, clientset *kubernetes.Clientset,
 	EventOnParent = strings.ToLower(eventParent)
 	recorder, _ = common.CreateRecorder(*KubeClient, controllerName)
 	config = kubeconfig
+	templates.InitializeKubeClient(kubeClient, kubeconfig)
 }
 
 //InitializeClient helper function to initialize kubeclient
@@ -307,6 +310,12 @@ func handleObjectTemplates(plc policyv1.ConfigurationPolicy, apiresourcelist []*
 	}
 	relatedObjects := []policyv1.RelatedObject{}
 	parentUpdate := false
+
+	// initialize apiresources for template processing before starting objectTemplate processing
+	// this is optional but since apiresourcelist is already available,
+	// use this rather than re-discovering the list for generic-lookup
+	templates.SetAPIResources(apiresourcelist)
+
 	for indx, objectT := range plc.Spec.ObjectTemplates {
 		nonCompliantObjects := map[string]map[string]interface{}{}
 		compliantObjects := map[string]map[string]interface{}{}
@@ -325,6 +334,38 @@ func handleObjectTemplates(plc policyv1.ConfigurationPolicy, apiresourcelist []*
 			glog.Error(jsonErr)
 			return
 		}
+
+		// Here appears to be a  good place to hook in template processing
+		// This is at the head of objectemplate processing
+		// ( just before the perNamespace handling of objectDefinitions)
+
+		// check here to determine if the object definition has a template
+		// and execute  template-processing only if  there is a template pattern "{{" in it
+		// to avoid unnecessary parsing when there is no template in the definition.
+
+		if templates.HasTemplate(string(ext.Raw)) {
+			resolvedblob, tplErr := templates.ResolveTemplate(blob)
+			if tplErr != nil {
+				update := createViolation(&plc, 0, "Error processing template", tplErr.Error())
+				if update {
+					recorder.Event(&plc, eventWarning, fmt.Sprintf(plcFmtStr, plc.GetName()), convertPolicyStatusToString(&plc))
+					addForUpdate(&plc)
+				}
+				return
+			}
+
+			//marshal it back and set it on the objectTemplate so be used  in processed further down
+			resolveddata, jsonErr := json.Marshal(resolvedblob)
+			if jsonErr != nil {
+				glog.Error(jsonErr)
+				return
+			}
+
+			//Set the resolved data for use in further processing
+			objectT.ObjectDefinition.Raw = resolveddata
+			blob = resolvedblob
+		}
+
 		unstruct.Object = blob.(map[string]interface{})
 		if md, ok := unstruct.Object["metadata"]; ok {
 			metadata := md.(map[string]interface{})
@@ -440,6 +481,7 @@ func createInformStatus(mustNotHave bool, numCompliant int, numNonCompliant int,
 	if kind == "" {
 		return
 	}
+
 	if mustNotHave {
 		if numNonCompliant > 0 { // We want no resources, but some were found
 			//noncompliant; mustnothave and objects exist
@@ -811,17 +853,17 @@ func getDetails(unstruct unstructured.Unstructured) (name string, kind string, n
 
 		metadata := md.(map[string]interface{})
 		if objectName, ok := metadata["name"]; ok {
-			name = objectName.(string)
+			name = strings.TrimSpace(objectName.(string))
 		}
 		// override the namespace if specified in objectTemplates
 		if objectns, ok := metadata["namespace"]; ok {
 			glog.V(5).Infof("overriding the namespace as it is specified in objectTemplates...")
-			namespace = objectns.(string)
+			namespace = strings.TrimSpace(objectns.(string))
 		}
 	}
 
 	if objKind, ok := unstruct.Object["kind"]; ok {
-		kind = objKind.(string)
+		kind = strings.TrimSpace(objKind.(string))
 	}
 	return name, kind, namespace
 }
@@ -852,7 +894,7 @@ func getNamesOfKind(unstruct unstructured.Unstructured, rsrc schema.GroupVersion
 	namespaced bool, ns string, dclient dynamic.Interface, complianceType string) (kindNameList []string) {
 	if namespaced {
 		res := dclient.Resource(rsrc).Namespace(ns)
-		resList, err := res.List(metav1.ListOptions{})
+		resList, err := res.List(context.TODO(), metav1.ListOptions{})
 		if err != nil {
 			glog.Error(err)
 			return kindNameList
@@ -860,7 +902,7 @@ func getNamesOfKind(unstruct unstructured.Unstructured, rsrc schema.GroupVersion
 		return buildNameList(unstruct, complianceType, resList)
 	}
 	res := dclient.Resource(rsrc)
-	resList, err := res.List(metav1.ListOptions{})
+	resList, err := res.List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		glog.Error(err)
 		return kindNameList
@@ -956,7 +998,7 @@ func getPolicyNamespaces(policy policyv1.ConfigurationPolicy) []string {
 func getAllNamespaces() (list []string) {
 	listOpt := &metav1.ListOptions{}
 
-	nsList, err := (*KubeClient).CoreV1().Namespaces().List(*listOpt)
+	nsList, err := (*KubeClient).CoreV1().Namespaces().List(context.TODO(), *listOpt)
 	if err != nil {
 		glog.Errorf("Error fetching namespaces from the API server: %v", err)
 	}
@@ -990,7 +1032,7 @@ func objectExists(namespaced bool, namespace string, name string, rsrc schema.Gr
 	exists := false
 	if !namespaced {
 		res := dclient.Resource(rsrc)
-		_, err := res.Get(name, metav1.GetOptions{})
+		_, err := res.Get(context.TODO(), name, metav1.GetOptions{})
 		if err != nil {
 			if errors.IsNotFound(err) {
 				glog.V(6).Infof("response to retrieve a non namespaced object `%v` from the api-server: %v", name, err)
@@ -1005,7 +1047,7 @@ func objectExists(namespaced bool, namespace string, name string, rsrc schema.Gr
 		}
 	} else {
 		res := dclient.Resource(rsrc).Namespace(namespace)
-		_, err := res.Get(name, metav1.GetOptions{})
+		_, err := res.Get(context.TODO(), name, metav1.GetOptions{})
 		if err != nil {
 			if errors.IsNotFound(err) {
 				exists = false
@@ -1032,7 +1074,7 @@ func createObject(namespaced bool, namespace string, name string, rsrc schema.Gr
 	if !namespaced {
 		res := dclient.Resource(rsrc)
 
-		_, err = res.Create(&unstruct, metav1.CreateOptions{})
+		_, err = res.Create(context.TODO(), &unstruct, metav1.CreateOptions{})
 		if err != nil {
 			if errors.IsAlreadyExists(err) {
 				created = true
@@ -1047,7 +1089,7 @@ func createObject(namespaced bool, namespace string, name string, rsrc schema.Gr
 		}
 	} else {
 		res := dclient.Resource(rsrc).Namespace(namespace)
-		_, err = res.Create(&unstruct, metav1.CreateOptions{})
+		_, err = res.Create(context.TODO(), &unstruct, metav1.CreateOptions{})
 		if err != nil {
 			if errors.IsAlreadyExists(err) {
 				created = true
@@ -1071,7 +1113,7 @@ func deleteObject(namespaced bool, namespace string, name string, rsrc schema.Gr
 	var err error
 	if !namespaced {
 		res := dclient.Resource(rsrc)
-		err = res.Delete(name, &metav1.DeleteOptions{})
+		err = res.Delete(context.TODO(), name, metav1.DeleteOptions{})
 		if err != nil {
 			if errors.IsNotFound(err) {
 				deleted = true
@@ -1084,7 +1126,7 @@ func deleteObject(namespaced bool, namespace string, name string, rsrc schema.Gr
 		}
 	} else {
 		res := dclient.Resource(rsrc).Namespace(namespace)
-		err = res.Delete(name, &metav1.DeleteOptions{})
+		err = res.Delete(context.TODO(), name, metav1.DeleteOptions{})
 		if err != nil {
 			if errors.IsNotFound(err) {
 				deleted = true
@@ -1176,14 +1218,30 @@ func mergeSpecsHelper(x1, x2 interface{}, ctype string) interface{} {
 }
 
 func mergeArrays(new []interface{}, old []interface{}, ctype string) (result []interface{}) {
+	if ctype == "mustonlyhave" {
+		return new
+	}
 	newCopy := append([]interface{}{}, new...)
 	indexesSkipped := map[int]bool{}
 	for i := range newCopy {
 		indexesSkipped[i] = false
 	}
-
+	oldItemSet := map[string]map[string]interface{}{}
 	for _, val2 := range old {
-		found := false
+		if entry, ok := oldItemSet[fmt.Sprint(val2)]; ok {
+			oldItemSet[fmt.Sprint(val2)]["count"] = entry["count"].(int) + 1
+		} else {
+			oldItemSet[fmt.Sprint(val2)] = map[string]interface{}{
+				"count": 1,
+				"value": val2,
+			}
+		}
+	}
+
+	for _, data := range oldItemSet {
+		count := 0
+		reqCount := data["count"]
+		val2 := data["value"]
 		for newIdx, val1 := range newCopy {
 			matches := false
 			if ctype != "mustonlyhave" {
@@ -1195,7 +1253,7 @@ func mergeArrays(new []interface{}, old []interface{}, ctype string) (result []i
 					mergedObj = val1
 				}
 				if reflect.DeepEqual(mergedObj, val2) && !indexesSkipped[newIdx] {
-					found = true
+					count = count + 1
 					matches = true
 				}
 				if matches && ctype != "mustonlyhave" && !indexesSkipped[newIdx] {
@@ -1204,12 +1262,14 @@ func mergeArrays(new []interface{}, old []interface{}, ctype string) (result []i
 				}
 
 			} else if reflect.DeepEqual(val1, val2) && !indexesSkipped[newIdx] {
-				found = true
+				count = count + 1
 				indexesSkipped[newIdx] = true
 			}
 		}
-		if !found {
-			new = append(new, val2)
+		if count < reqCount.(int) {
+			for i := 0; i < (reqCount.(int) - count); i++ {
+				new = append(new, val2)
+			}
 		}
 	}
 	return new
@@ -1308,6 +1368,14 @@ func handleSingleKey(key string, unstruct unstructured.Unstructured, existingObj
 			if oldObj == nil || !checkFieldsWithSort(mergedObj, oldObj.(map[string]interface{})) {
 				updateNeeded = true
 			}
+		case ([]map[string]interface{}):
+			if oldObj == nil || !checkListFieldsWithSort(mergedObj, oldObj.([]map[string]interface{})) {
+				updateNeeded = true
+			}
+		case ([]interface{}):
+			if oldObj == nil || !checkListsMatch(mergedObj, oldObj.([]interface{})) {
+				updateNeeded = true
+			}
 		default:
 			if !reflect.DeepEqual(nJSON, oJSON) {
 				updateNeeded = true
@@ -1349,7 +1417,7 @@ func handleKeys(unstruct unstructured.Unstructured, existingObj *unstructured.Un
 			}
 			//enforce
 			glog.V(4).Infof("Updating %v template `%v`...", typeStr, name)
-			_, err = res.Update(existingObj, metav1.UpdateOptions{})
+			_, err = res.Update(context.TODO(), existingObj, metav1.UpdateOptions{})
 			if errors.IsNotFound(err) {
 				message := fmt.Sprintf("`%v` is not present and must be created", typeStr)
 				return false, false, message, true
@@ -1380,7 +1448,7 @@ func updateTemplate(
 	} else {
 		res = dclient.Resource(rsrc)
 	}
-	existingObj, err := res.Get(name, metav1.GetOptions{})
+	existingObj, err := res.Get(context.TODO(), name, metav1.GetOptions{})
 	if err != nil {
 		glog.Errorf(getObjError, name)
 	} else {
