@@ -1,4 +1,5 @@
 // Copyright (c) 2020 Red Hat, Inc.
+// Copyright Contributors to the Open Cluster Management project
 
 package configurationpolicy
 
@@ -8,8 +9,8 @@ import (
 	"path/filepath"
 	"testing"
 
-	"github.com/stolostron/config-policy-controller/pkg/apis"
-	policiesv1alpha1 "github.com/stolostron/config-policy-controller/pkg/apis/policy/v1"
+	"github.com/open-cluster-management/config-policy-controller/pkg/apis"
+	policiesv1alpha1 "github.com/open-cluster-management/config-policy-controller/pkg/apis/policy/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/client-go/kubernetes/scheme"
